﻿using Nemerle;
using Nemerle.Collections;
using Nemerle.Text;
using Nemerle.Utility;
using Nemerle.Peg;

using System;
using System.Collections.Generic;
using System.Linq;
using NuSoft.ExpressionEditor.Common.Interfaces;

namespace NuSoft.ExpressionEditor.ExampleParser
{
  /// <summary>
  /// Description of MathParser.
  /// </summary>
    [PegGrammar( Options = EmitDebugSources, Start, 
    grammar{
        
        Space   :void = ' '*;
        POpen   :void = '(' Space;
        PClose  :void = ')' Space;    
        
        
        Digits        :int = ['0'..'9']+ Space;
        PropertyName  :int = '$'['a'..'z', 'A'..'Z'] ['a'..'z', 'A'..'Z','0'..'9']* Space;
        LocalVar      :string = ['a'..'z', 'A'..'Z'] ['a'..'z', 'A'..'Z','0'..'9']* Space;
        LocalValue    :int = LocalVar;
        Value         :int = Digits / PropertyName ;// LocalValue;
        
        //Let           :object = "let" Space LocalVar "=" Space Expr20 "in" Space;
        
        
        AbsExpr       :int = "Abs(" Space Expr20 ")" Space;
        
        
        
        UnaryExpr     :int = '-' Space Term;
        Parentheses   :int = POpen Expr20 PClose;
        Quotient      :BinaryOperation*int = '/' Space Term;// BinaryOp;
        Sum           :BinaryOperation*int = '+' Space Expr10;// BinaryOp;
        Difference    :BinaryOperation*int = '-' Space Expr10;// BinaryOp;
        Product       :BinaryOperation*int = '*' Space Term;// BinaryOp
        BinaryOp20    :BinaryOperation*int = Difference / Sum / Product / Quotient;
        BinaryOp10    :BinaryOperation*int = Product / Quotient;
        Expr20        :int = /*Let* */ Term BinaryOp20*;
        Expr10        :int = Term BinaryOp10*;
        Term          :int = UnaryExpr / Parentheses / AbsExpr / Value;// / Value / Sum ;
        Start         :int = Space Expr20 ![Any];
        
        //let x = 
  })]
  public class MathParser:IParser
  {
      supportedMethods : array[ISupportedMethod];
      locals:Dictionary[string,int] = Dictionary();
      
      public Parse(obj:object,expr:string):IParsingResult{
          this.sourceObj=obj;
          this.locals.Clear();
          try{
              def (p,r) = this.TryParse(expr);
              if(p==expr.Length) {
                  MathParserResult(r,null)
              }
              else {
                  MathParserResult(0,"Unknown parsing error")
              }
          }
          catch{
              | e => MathParserResult(0,e.Message)
          }
      }
      
      public this(){
          supportedMethods = array[SupportedMethod("Abs","Abs(EXPR)","Returns absolute value of expression")];   
      }
      
      public SupportedMethods:IEnumerable[ISupportedMethod]{
          get{supportedMethods}
      }
       
      private AbsExpr(_:NToken,num:int,_:NToken):int{
          def res =Math.Abs(num);
          res
      }
      
      private Digits(digits:NToken):int{
          def text=GetText(digits);
          def res = int.Parse(text);
          res
      }
      
      private Expr20(/*_:List[object],*/a:int,l:List[BinaryOperation*int]):int {
          mutable res = a;
          foreach(x in l){
              res = DoBinaryOp(res,x[0],x[1])
          }
          
          res
      }
      
      private Expr10(a:int,l:List[BinaryOperation*int]):int {
          mutable res = a;
          foreach(x in l){
              res = DoBinaryOp(res,x[0],x[1])
          }
          
          res
      }
      
      private DoBinaryOp(a:int,op:BinaryOperation,b:int):int{
          match(op){
              | BinaryOperation.Addition => a+b
              | BinaryOperation.Multiplication => a*b
              | BinaryOperation.Substraction => a-b
              | BinaryOperation.Division => a/b
          }
      }
      
      private UnaryExpr(_:NToken,num:int):int{
          -num
      }
      
      private Unary(num:int):int{
          num
      }
      
      private Num(num:int):int{
          num;
      }
      
      
      private Parentheses(num:int):int{
          num;
      }
      
      private Sum(_:NToken,num:int):BinaryOperation*int{
          (BinaryOperation.Addition,num)
      }
      
      private Difference(_:NToken,num:int):BinaryOperation*int{
          (BinaryOperation.Substraction,num)
      }
      private Product(_:NToken,num:int):BinaryOperation*int{
          (BinaryOperation.Multiplication,num)
      }
      private Quotient(_:NToken,num:int):BinaryOperation*int{
          (BinaryOperation.Division,num)
      }
      
      private PropertyName(_:NToken,p1:NToken,p2:NToken):int{
          def propName = GetText(p1)+GetText(p2);
          def type =this.sourceObj.GetType();
          def prop = type.GetProperty(propName);
          prop.GetValue(this.sourceObj) :> int
      }
      
      private LocalVar(p1:NToken,p2:NToken):string{
          GetText(p1)+GetText(p2)
      }
      
      public LocalValue(varName:string):int{
          this.locals[varName]
      }
      
      private Let(_:NToken,varDef:string,_:NToken,varValue:int,_:NToken):object{
          this.locals.Add(varDef,varValue);
          null
      }
      
      private mutable sourceObj:object;
      
  }
  
}

